function [ S1, S2 ] = StableRegionWrapper( filename1, filename2 )
% StableRegionWrapper is a wrapper function which computes the necessary
% descriptors and runs the stable region process for the given shapes.
%
% Input: The file names of two .off files (without the .off extension).
% Output: S1 and S2 are shape structure. Each contains the following
% fields:
%       .surface = a structure containing X, Y, Z coordinates,
%           and TRIV (a list of triangular faces represented by vertex ids).
%       .stab = raw output of the stable region process, a binary matrix
%           in which vertices of the same stable region have a value of 1 
%           in the same column.
%       .seg = a vector which denotes the region id of each vertex.
% 
% Before running this function MAKE SURE THAT THE path is set correctly.
%
% Wrapper function written by Yanir Kleiman, 2017.
%
% Original stable region code provided by authors of:
% "Stable Region Correspondences Between Non-Isometric Shapes" 
% [Ganapathi-Subramanian et al 2016].
    
    %%% Compute descriptors:
    numEig = 100;

    % Load mesh and computes laplacian, areaweights, cotweights, etc.
    mesh1 = loadMeshLB(filename1, numEig);
    mesh2 = loadMeshLB(filename2, numEig);

    % descriptor options
    options.descriptors = {};
    options.descriptorsParams = {};

    % WKS
    options.descriptors{end+1}          = 'wks';
    wksParams.timeSteps = 100;
    options.descriptorsParams{end+1}    = wksParams;       

    % HKS
    options.descriptors{end+1}          = 'hks';
    hksParams.timeSteps = 100;
    options.descriptorsParams{end+1}    = hksParams;       

    % Smoothed Mean Curvature
    options.descriptors{end+1}          = 'mcs';
    mcsParams.timeSteps = 100;
    options.descriptorsParams{end+1}    = mcsParams;       


    % Compute descriptors according to above descriptor options:
    fprintf(1, 'Computing descriptors...\n');
    [f, g] = collectDescriptors(mesh1, mesh2, options);

    S1.surface = struct();
    S1.surface.X = mesh1.vertices(:, 1);
    S1.surface.Y = mesh1.vertices(:, 2);
    S1.surface.Z = mesh1.vertices(:, 3);
    S1.surface.nv = size(mesh1.vertices, 1);
    S1.surface.TRIV = double(mesh1.triangles);

    S2.surface = struct();
    S2.surface.X = mesh2.vertices(:, 1);
    S2.surface.Y = mesh2.vertices(:, 2);
    S2.surface.Z = mesh2.vertices(:, 3);
    S2.surface.nv = size(mesh2.vertices, 1);
    S2.surface.TRIV = double(mesh2.triangles);


    % Computing stable regions:
    [stab1, stab2] = stable_region_corrs(f, S1, mesh1.areaWeights, g, S2, mesh2.areaWeights, 3);

    S1.stab = stab1;
    S2.stab = stab2;
    
    S1.seg = zeros(length(mesh1.vertices), 1);
    S2.seg = zeros(length(mesh2.vertices), 1);

    m = size(stab1, 2); % number of columns = number of stable regions
    for j=1:m
        S1.seg(stab1(:, j) == 1) = j;
        S2.seg(stab2(:, j) == 1) = j;
    end;

end

